"""This module is the main module of the package and is used to interact with the Chegg API"""
from os import environ
from requests import Session, RequestException
from .util import welcome_banner, random_wait, play_success_notification
from .login_util import login_user
from .logout_util import logout_user
from .question_util import fetch_latest_question, skip_latest_question, analyze_question
from .question import Question

__version__ = "1.0.1"


class CheggPy:
    """
        A class to interact with the Chegg API.

        Args:
            username (str): The username or email of the user.
            password (str): The password of the user.
            keywords (list): Keywords to search for in a chegg question to answer.
            short_timeout (tuple): A tuple of two integers representing the range of short timeouts.For example, (5, 10) means the timeout will be between 5 and 10 seconds. This is used for the time between subsequent API requests.
            long_timeout (tuple): A tuple of two integers representing the range of long timeouts. For example, (5*60, 10*60) means the timeout will be between 5 and 10 minutes. This is used to wait when there are no questions to answer.

        Returns:
            requests.Session: The session object after logging in.
    """

    DASHBOARD_URL = "https://expert.chegg.com/qna/authoring/answer"

    def __init__(self,
                 username=None,
                 password=None,
                 keywords=None,
                 short_timeout=(5, 10),
                 long_timeout=(5*60, 10*60)
                 ):
        welcome_banner()
        self.username = username or environ.get('CHEGG_USER')
        self.password = password or environ.get('CHEGG_PW')
        self.keywords = keywords or []
        self.session = Session()
        self.latest_question = Question()
        self.short_timeout = short_timeout
        self.long_timeout = long_timeout

    def login(self):
        """Login the user with the provided username and password"""
        try:
            self.session = login_user(
                self.username, self.password, self.session)
            print('Logged in successfully..........')
            random_wait(*self.short_timeout)
        except AssertionError as err:
            print(str(err))
            raise
        except RequestException as err:
            print('Login failed:', err)
            raise
        return self

    def logout(self):
        """Logout the user"""
        try:
            logout_user(self.session)
            print('Logged out successfully..........')
        except NotImplementedError as err:
            print(str(err))

    def fetch_question(self):
        """Fetch the latest question"""
        try:
            self.latest_question = Question(
                fetch_latest_question(self.session))
            print('Fetched the question successfully with id:',
                  self.latest_question.id)
            random_wait(*self.short_timeout)
        except NotImplementedError as err:
            print(str(err))
        return self

    def skip_question(self):
        """Skip the latest question"""
        try:
            skip_latest_question(self.session, self.latest_question)
            print('Skipped the question successfully with id:',
                  self.latest_question.id)
            random_wait(*self.short_timeout)
        except NotImplementedError as err:
            print(str(err))
        return self

    def is_question_answerable(self):
        """Check if the latest question is answerable"""
        if analyze_question(self.latest_question, self.keywords):
            play_success_notification()
            return True
        return False



"""The module that handles a question and its properties of Chegg"""
from bs4 import BeautifulSoup


class Question:
    """Class to represent a question in Chegg"""

    def __init__(self, response=None):
        response = response or {}
        question = response.get('data', {}).get(
            'nextQuestionAnsweringAssignment', {}).get('question', {})
        self.id = question.get('id')
        self.uuid = question.get('uuid')
        self.body = self.__sanitize_html(question.get('body', "") or "")
        self.transcription = self.__sanitize_html(
            question.get('imageTranscriptionText', "") or "")

    def __str__(self):
        return f"Body: {self.body}\nTranscription: {self.transcription}"

    def __sanitize_html(self, html_data):
        return BeautifulSoup(html_data, 'html.parser').get_text()



"""Module only used for the logged out part of the script"""


def logout_user(session):
    """Logins the user with the given username and password"""

    chegg_logout_url = "https://expert.chegg.com/api/auth/logout"
    session.request("POST", chegg_logout_url)
    session.close()



"""Module only used for the login part of the script"""
import json


def login_user(username, password, session):
    """
    Logins the user with the given username and password.

    Args:
        username (str): The username or email of the user.
        password (str): The password of the user.
        session (requests.Session): The session object to use for the request.

    Returns:
        requests.Session: The session object after logging in.
    """
    assert username, "Username not provided"
    assert password, "Password not provided"

    chegg_login_url = "https://expert.chegg.com/api/auth/login"
    payload = json.dumps({
        "email": username,
        "password": password
    })
    headers = {
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/121.0.0.0 Safari/537.36',
        'Content-Type': 'application/json',
        'Accept': 'application/json, text/plain, */*',
        'sec-ch-ua-platform': '"macOS"',
    }

    response = session.post(chegg_login_url, headers=headers, data=payload)
    response.raise_for_status()  # Raise exception for non-2xx status codes
    return session



"""A simple script to demonstrate the usage of CheggPy."""
from cheggpy import CheggPy
chegg = CheggPy(username='username',
                password='password',
                keywords=['python', 'java', 'c++'],
                ).login()
try:
    while True:
        chegg.fetch_question()
        if chegg.is_question_answerable():
            skip = input(
                "Press enter to continue or press any other key to stop the script.")
            if skip == "":
                chegg.skip_question()
            else:
                raise KeyboardInterrupt
        else:
            chegg.skip_question()
except KeyboardInterrupt:
    print("Script stopped by user..........")
    chegg.logout()



# __main__.py

import argparse
import sys
from cheggpy import CheggPy
from .util import goodbye_banner
from requests.exceptions import HTTPError


def parse_args():
    """Parse the command line arguments"""
    parser = argparse.ArgumentParser()
    parser.add_argument("-u", "--username", help="Your Chegg username", required=True)
    parser.add_argument("-p", "--password", help="Your Chegg password", required=True)
    parser.add_argument("-k", "--keywords", help="Keywords to search for in a chegg question to answer", nargs="+", required=True)
    parser.add_argument("-st", "--short_timeout", help="A tuple of two integers representing the range of short timeouts. Default, (5, 10) means the timeout will be between 5 and 10 seconds. This is used for the time between subsequent API requests.", nargs=2, type=int, default=[5, 10])
    parser.add_argument("-lt", "--long_timeout", help="A tuple of two integers representing the range of long timeouts. Default, (5, 10) means the timeout will be between 5 and 10 minutes. This is used to wait when there are no questions to answer.", nargs=2, type=int, default=[5*60, 10*60])

    args = parser.parse_args()
    return args


def main():
    """Run the script to fetch and start ananlyzing the latest question"""
    args = parse_args()
    chegg = CheggPy(username=args.username,
                    password=args.password,
                    keywords=args.keywords,
                    short_timeout=args.short_timeout,
                    long_timeout=args.long_timeout
                    )
    try:
        chegg.login()
        while True:
            chegg.fetch_question()
            if chegg.is_question_answerable():
                print("Hurray!! You got a suitable question to answer -> {}".format(CheggPy.DASHBOARD_URL))
                skip = input(
                    "Press enter to continue or press any other key to stop the script.")
                if skip == "":
                    chegg.skip_question()
                else:
                    raise KeyboardInterrupt
            else:
                chegg.skip_question()
    except HTTPError as e:
        print(e)
    except KeyboardInterrupt:
        print("Script stopped by user..........")
    finally:
        chegg.logout()
        goodbye_banner()
        sys.exit(0)


if __name__ == "__main__":
    main()



"""This module contains utility functions for the question on chegg."""

import json


def fetch_latest_question(session):
    '''
        Fetch the latest question in the queue.
        Returns dict with question_id, body, etc. if question is present.
        Otherwise, returns None or raises an Exception.
        '''
    # log('Fetching the question')
    chegg_question_url = "https://gateway.chegg.com/nestor-graph/graphql"
    payload = json.dumps({
        "operationName": "NextQuestionAnsweringAssignment",
        "variables": {},
        "query": "query NextQuestionAnsweringAssignment {\n  nextQuestionAnsweringAssignment {\n    question {\n      body\n      id\n      uuid\n      subject {\n        id\n        name\n        subjectGroup {\n          id\n          name\n          __typename\n        }\n        __typename\n      }\n      imageTranscriptionText\n      lastAnswerUuid\n      __typename\n    }\n    langTranslation {\n      body\n      translationLanguage\n      __typename\n    }\n    questionGeoLocation {\n      countryCode\n      countryName\n      languages\n      __typename\n    }\n    questionRoutingDetails {\n      answeringStartTime\n      bonusCount\n      bonusTimeAllocationEnabled\n      checkAnswerStructureEnabled\n      hasAnsweringStarted\n      questionAssignTime\n      questionSolvingProbability\n      routingType\n      allocationExperimentId\n      questionQualityFactor\n      __typename\n    }\n    __typename\n  }\n}"
    })
    headers = {
        'authority': 'gateway.chegg.com',
        'accept': '*/*',
        'accept-language': 'en-GB,en;q=0.9',
        'apollographql-client-name': 'chegg-web-producers',
        'content-type': 'application/json',
        'origin': 'https://expert.chegg.com',
        'referer': 'https://expert.chegg.com/',
        'sec-fetch-dest': 'empty',
        'sec-fetch-mode': 'cors',
        'sec-fetch-site': 'same-site',
        'sec-gpc': '1',
        'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36'
    }

    session.headers.update(headers)
    response = json.loads(session.request(
        "POST", chegg_question_url, data=payload).text)
    if response.get('errors') is not None:
        raise NotImplementedError(response['errors'][0]['message'])
    return response


def skip_latest_question(session, question):
    """Skip the latest question in the queue."""
    if question is None or question.id is None:
        raise NotImplementedError('No question to skip')
    chegg_question_url = "https://gateway.chegg.com/nestor-graph/graphql"
    payload = json.dumps({
        "operationName": "SkipQuestionAssignment",
        "variables": {
            "questionId": question.id,
            "skipPageFlow": "DECISION",
            "skipPrimaryReason": {
                "noKnowledge": True
            }
        },
        "query": "mutation SkipQuestionAssignment($questionId: Long!, $skipPageFlow: QnaCurrentPageFlow!, $skipPrimaryReason: QuestionSkipPrimaryReasons, $newSkipReason: QuestionNewSkipReasons) {\n  skipQuestionAssignment(\n    questionId: $questionId\n    skipPageFlow: $skipPageFlow\n    skipPrimaryReason: $skipPrimaryReason\n    newSkipReason: $newSkipReason\n  ) {\n    message\n    questionId\n    __typename\n  }\n}"
    })
    response = json.loads(session.request(
        "POST", chegg_question_url, data=payload).text)
    if response.get("errors") is not None:
        raise NotImplementedError(response['errors'][0]['message'])


def analyze_question(question, keywords):
    """Analyze the question and return the result"""
    ques_text = question.body + ' ' + question.transcription
    if any(word in ques_text for word in keywords):
        return True
    return False
